import board
import random
import neopixel
from analogio import AnalogIn
import digitalio
# slider potentiometer in A3
knob = AnalogIn(board.A3)

# buttonOne in pin GP7
buttonOne = digitalio.DigitalInOut(board.GP7)
buttonOne.switch_to_input(pull=digitalio.Pull.UP)
# set a flag to capture if the button is pressed or not to False
butFlag = False

#default settings for NeoPixel
BRIGHTNESS = 1.0
ORDER = neopixel.GRB

# create a (neo)"pixel" object using pin A0
pixels = neopixel.NeoPixel(
    board.A0, 1, brightness=1, auto_write=False, pixel_order=ORDER
)

# a dict to keep track of colours (base settings + possible mix values)
# see addTuples(a,b) function to add two tuples together to create a new R,G,B value
colPairs = {
    'red': {'base': (255, 0,0), 'mix': (0, 25, 25)},
    'blue': {'base': (0, 0,255), 'mix': (10, 55, 0)},
    'yellow': {'base': (200, 200, 50), 'mix': (55, 55, 10)},
    'green': {'base': (0, 255, 0), 'mix': (25, 0, 50)},
    'orange' : {'base':(200,0,0), 'mix': (55, 100, 50)},
    'pink' : {'base':(255,0,50), 'mix': (0, 100, 50)},
    'rand' : {'base':(10,10,10), 'mix': (245, 245, 245)},
    'slider' : {'base' : (0,0,0), 'mix' : (0,0,0)}
}
# pull the Keys from the dict into a standard list / array
keys = list(colPairs.keys())

# utilities functions 
def clamp(n, minn, maxn):
    # clamps a value (n) between min / max passed
    return max(min(maxn, n), minn)

def addTuples(a, b):
    # adds two tuples together (50, 50, 55) + (125, 125, 200) = (175, 175, 255)
    return(tuple(map(sum, zip(a, b))))

def shuffle(l, n):
    # Shuffle a list by "n" (- or +)
    return(l[n:] + l[:n])

def mapFromTo(v, oldMin, oldMax, newMin, newMax):
    # map a value in a range to a new range e.g. (100, 0, 200, 0, 100) = 50
    newV = (v-oldMin)/(oldMax-oldMin)*(newMax-newMin)+newMin
    return newV

def mixCols(base, mix):
    # use addTuples & set the pixel[0] value
    r,g,b = addTuples(base,mix)
    # apply the mix to the neoPixel
    pixels[0] = (r,g,b)
    # you need a "show" to update the neoPixel
    pixels.show()    

def wheel(pos):
    # from adafruit
    # Input a value 0 to 255 to get a color value.
    # The colours are a transition r - g - b - back to r.
    # Phil converts the slider value from its own range to 0-255 via mapFromTo()
    if pos < 0 or pos > 255:
        r = g = b = 0
    elif pos < 85:
        r = int(pos * 3)
        g = int(255 - pos * 3)
        b = 0
    elif pos < 170:
        pos -= 85
        r = int(255 - pos * 3)
        g = 0
        b = int(pos * 3)
    else:
        pos -= 170
        r = 0
        g = int(pos * 3)
        b = int(255 - pos * 3)
    return (r, g, b) if ORDER in (neopixel.RGB, neopixel.GRB) else (r, g, b, 0)

# some variables for home made "sleep" without creating inactivity
wait = 500
count = 0

print("starting with {}".format(keys[0]))

while True:
    # add up the counter variable
    count += 1
    # clamp the knob value to 130 / 65200
    clampedKnob = clamp(knob.value, 130, 65200)
    # create "mixRed, mixGreen, mixBlue" values based on the slider knob position & the possible "mix" maximums
    mR = int(mapFromTo(clampedKnob, 130,
                                 65200, 0, random.randint(0,colPairs[keys[0]]['mix'][0])))
    mG = int(mapFromTo(clampedKnob, 130,
                                 65200, 0, random.randint(0,colPairs[keys[0]]['mix'][1])))
    mB = int(mapFromTo(clampedKnob, 130,
                                 65200, 0, random.randint(0, colPairs[keys[0]]['mix'][2])))
    # if the button is pressed (and the flag is not "on")
    if not buttonOne.value and not butFlag:
        # shuffle the keys by +1
        keys = shuffle(keys, 1)
        #show what the new 1st position in keys is...
        print(keys[0])
        # mix the colours from the dict "colPairs" (base + (mR, mG, mB))
        mixCols(colPairs[keys[0]]['base'], (mR, mG, mB))
        # set the button flag = True (to stop the colour change happening all the)
        butFlag = True
        
    if buttonOne.value and butFlag:
        # if the button has been released & the flag was true set the flag to false (reset button to be pressable)
        butFlag = False
        
    # get the mixRed,green,blue from the Slider & current mixTuple if key isn't "slider"
    if keys[0] != 'slider':
        if count >= wait:
            mixCols(colPairs[keys[0]]['base'], (mR, mG, mB))
            count = 0
    else:
        # if the key is "slider" we use the slider to change the colours using the wheel function 
        count = 0
        # conver the slder value to something between 0 & 255
        # update the neoPixel
        pixels[0] = wheel(int(mapFromTo(clampedKnob, 130,65200, 0,255)))
        pixels.show()
        
